/* genlbtab.c  2006/03/26             */
/* MIDI volume (0..127) to log2 table */
/* log2 value to linear table         */
/* NON COMMERCIAL USE ONLY            */
/* 2011/04/25 added PAN (CC#10) table */

#include <stdio.h>
#include <math.h>

#define ITEMPERLINE  8
#define LBTABSIZE   64
#define LBFRACS     64
#define MIDIVOLS   128
#define PANVALS    128

int main()
{
  int i, hex;
  double lin, lb, pan_l;
  double dB_6 = 20.0 * log10(2.0); // 2 = 6.02 dB
  printf("#include \"../common/tonegen.h\"\r\xa");
  printf("\r\xa");
  printf("// convert log2 value to linear value\r\xa");
  printf("// LIN = lb2lin[LB] = 2**(-LB/%d)\r\xa\r\xa", LBFRACS);
  printf("const int16_t lb2lin[%d] = {", LBTABSIZE);
  for (i = 0; i < LBTABSIZE; i++) {
//    dB = -(dB_6/(double)LBFRACS) * i;   // 0.09375 dB step
//    lin = pow(10.0, dB / 20.0);
    lin = pow(2.0, -i/(double)LBFRACS);
    hex = round(32767.0 * lin);
    if (0 == (i % ITEMPERLINE)) {
      printf("\r\xa  ");
    }
    printf("0x%.4x", hex);
    printf("%s", (((LBTABSIZE-1) == i) ? "" : ", "));
  }
  printf("\r\xa}; // const int16_t lb2lin[%d]\r\xa", LBTABSIZE);

  printf("\r\xa// General MIDI volume value to log2 value");
  printf("\r\xa// LB = mv2lb[VOL] = -%d*log2((VOL/127)**2)\r\xa\r\xa", LBFRACS);
  printf("const int16_t mv2lb[%d] = {",MIDIVOLS);
  for (i = 0; i < MIDIVOLS; i++) {
    if (0 == i) hex = 0x3ff;
    else {
      lb = -2.0*log(i/127.0)/log(2.0); /* Genral MIDI */
      hex = round(LBFRACS * lb);
    } /* if */
    if (0 == (i % ITEMPERLINE)) {
      printf("\r\xa  ");
    }
    printf("0x%.4x", hex);
    printf("%s", ((127 == i) ? "" : ", "));
  }
  printf("\r\xa}; // const int16_t mv2lb[%d]\r\xa", MIDIVOLS);
  
  printf("\r\xa#if (STEREO_MODE)\r\xa");
  printf("\r\xa// Control Change #10 (PAN) value to linear value");
  printf("\r\xa// using GM2 recommended formula");
  printf("\r\xa// cc10_tab[PAN] = cos((M_PI/2) * max(0, PAN-1) / 126)\r\xa\r\xa");
  printf("const int16_t cc10_tab[%d] = {",PANVALS);
  for (i = 0; i < PANVALS; i++) {
    if (0 == i) hex = 32767;
    else {
      pan_l = cos((M_PI/2.0) * (i-1) / 126.0); // GM2 recommendation
      hex = round(32767.0 * pan_l);
    } /* if */
    if (0 == (i % ITEMPERLINE)) {
      printf("\r\xa  ");
    }
    printf("0x%.4x", hex);
    printf("%s", ((127 == i) ? "" : ", "));
  }
  printf("\r\xa}; // const int16_t cc10_tab[%d]\r\xa", PANVALS);
  printf("\r\xa#endif // #if (STEREO_MODE)\r\xa");
  return(0);
}
