/* gen_rx62conf.c                       2011-Apr-03   */
/* generate system configuration table for RX62N      */
/* 2011/05/04 for stereo/monaural                     */

#include <stdio.h>
#include <math.h>

#define MAX_CONFIG 5

char t1ms_tab[2][MAX_CONFIG] = {
  {48, 40, 32, 30, 24}, // for stereo
  {48, 40, 32, 30, 24}  // for monaural
};

char poly_tab[2][MAX_CONFIG] = {
 {13, 15, 20, 22, 27}, // for stereo
 {15, 18, 24, 25, 32}, // for monaural
};

int tab_size[2] = {4, 4};

/*
double log2(double x)
{
  return (log(x) / log(2.0));
} // double log2()
*/

int main()
{
  int i, j, t1ms, t4ms, t1div, oct_ofs, note_ofs, n_poly;
  int Q_phacc   = 28;
  int Q_fnumber = 15;
  double min_MIDI_freq = 440.0 * pow(2, -69.0/12.0);
  double fs, lb_fs, ofs;
  double hclk = 48e6;
  double lb_base = (log2(min_MIDI_freq) + Q_phacc - Q_fnumber + 1);
  printf("/***********************************************/\r\xa");
  printf("/*  rx62conf.c : system configuration table    */\r\xa");
  printf("/***********************************************/\r\xa");

  printf("\r\xa");
  printf("#include \"../common/tonegen.h\"\r\xa");
  printf("\r\xa");
  printf("// PCLK    = %6.3f MHz\r\xa", hclk/1e6);
  printf("\r\xa");
  printf("// lb_ofs  = log2(min_MIDI_freq) - log2(fs) + Q_phacc - Q_fnumber + 1\r\xa");
  printf("//         = %9.6f - log2(fs)\r\xa", lb_base);
//  printf("//         = %9.6f - log2(20000000 / t1div)\r\xa", lb_base);
  printf("//         = log2(t1div) - %8.6f\r\xa", -lb_base + log2(hclk));
  printf("\r\xa");
  printf("// oct_ofs  = floor(lb_ofs)\r\xa");
  printf("\r\xa");
  printf("// note_ofs = 1536 * (lb_ofs - oct_ofs)\r\xa");
  printf("\r\xa");
  for (j = 0; j < 2; j++) { //
    if (0 == j) {
      printf("const sys_config_st_t sys_config[2][MAX_CONFIG] = {\r\xa");
    }
    printf("//\r\xa");
    printf("//\r\xa");
    printf(" {// poly, t1div, tick1ms, tick4ms, oct_ofs, note_ofs, fs_str}\r\xa");
    for (i = 0; i < MAX_CONFIG; i++) {
      t1ms   = t1ms_tab[j][i];
      n_poly = poly_tab[j][i];
      t4ms = 4 * t1ms;
      fs = 1e3 * t1ms;
      t1div = 2 * round(hclk / (2.0 * fs));
      fs = hclk / (t1div);
      ofs = (lb_base - log2(fs));
      oct_ofs = floor(ofs);
      note_ofs = round(1536.0 * (ofs - oct_ofs));
      printf("   {%5d, %5d, %7d, %7d, %7d, %8d,  \"%c%c\" }", 
                n_poly, t1div, t1ms, t4ms, oct_ofs, note_ofs,
                ((t1ms / 10) ? '0'+(t1ms / 10) : ' '), 
                '0'+(t1ms % 10));
      if (sizeof(t1ms_tab)-1 > i) printf(","); else printf(" ");
      printf("// fs=%6.3f kHz\r\xa", fs * 1e-3); 
    } // for (i = 
    printf(" }");
    if (0 == j) {
      printf(",\r\xa");
    }
    else {
      printf("\r\xa}; // const sys_config_st_t sys_config[]\r\xa");
    }
  } // for (j
  return(0);
} // int main()
