/* genconfig.c,                     2010-May-13     */
/* generate system configuration table for SH7262 */

#include <stdio.h>
#include <math.h>

#define MAX_CONFIG 7

double ifs_code_tab[16] = { // SRC input fs code
   8.0,   //  0 : 8 kHz
  11.025, //  1 : 11.025 kHz
  12.0,   //  2 : 12 kHz
   0.0,   //  3 : illegal
  16.0,   //  4 : 16 kHz
  22.05,  //  5 : 22.05 kHz
  24.0,   //  6 : 24 kHz
   0.0,   //  7 : illegal
  32.0,   //  8 : 32 kHz
  44.1,   //  9 : 44.1 kHz
  48.0,   // 10 : 48 kHz
   0.0,   // 11 : illegal
   0.0,   // 12 : illegal
   0.0,   // 13 : illegal
   0.0,   // 14 : illegal
   0.0    // 15 : illegal
};
  

double fs_tab[2][MAX_CONFIG] = {
  {44.1, 44.1, 32.0, 24.0, 22.05, 16.0, 12.0}, // for slow chip
  {48.0, 44.1, 32.0, 24.0, 22.05, 16.0, 12.0}  // for fast chip
};

char poly_tab[2][MAX_CONFIG] = {
  { 11,  11,  16,  16,  16,  16,  16},
  { 10,  10,  16,  16,  16,  16,  16}
};

int tab_size[2] = {4, 4};

/*
double log2(double x)
{
  return (log(x) / log(2.0));
} // double log2()
*/

int main()
{
  int i, j, t1ms, t4ms, fs_code, oct_ofs, note_ofs;
  int Q_phacc   = 28;
  int Q_fnumber = 15;
  double min_MIDI_freq = 440.0 * pow(2, -69.0/12.0);
  double fs, lb_fs, ofs;
  double hclk = 24.0e6;
  double lb_base = (log2(min_MIDI_freq) + Q_phacc - Q_fnumber + 1);
  printf("/***********************************************/\r\xa");
  printf("/*  sysconf.c : system configuration table     */\r\xa");
  printf("/*              for SH-2A/FPU                  */\r\xa");
//  printf("/*              counter clock = %4.1f MHz       */\r\xa", hclk/1e6);
  printf("/***********************************************/\r\xa");

  printf("\r\xa");
  printf("#include \"../common/tonegen.h\"\r\xa");
  printf("\r\xa");
  printf("// lb_ofs  = log2(min_MIDI_freq) - log2(fs) + Q_phacc - Q_fnumber + 1\r\xa");
  printf("//         = %9.6f - log2(fs)\r\xa", lb_base);
//  printf("//         = %9.6f - log2(%4.1fe6 / t1div)\r\xa", lb_base, hclk/1e6);
//  printf("//         = log2(t1div) - %8.6f\r\xa", -lb_base + log2(hclk));
  printf("\r\xa");
  printf("// oct_ofs  = floor(lb_ofs)\r\xa");
  printf("\r\xa");
  printf("// note_ofs = 1536 * (lb_ofs - oct_ofs)\r\xa");
  printf("\r\xa");
  for (j = 0; j < 2; j++) { //
    if (0 == j) {
      printf("const sys_config_st_t sys_config[2][MAX_CONFIG] = {\r\xa");
    }
    printf("//\r\xa");
    printf("// %s %s\r\xa", 
           "for output fs = ", (j ? "48 kHz" : "44.1 kHz"));
    printf("//\r\xa");
    printf(" {// poly, fscode, tick1ms, tick4ms, oct_ofs, note_ofs, fs_str}\r\xa");
    for (i = 0; i < MAX_CONFIG; i++) {
      fs   = fs_tab[j][i];
      t1ms = (int)fs;
      t4ms = 4 * t1ms;
      ifs_code_tab[15] = fs;
      fs_code = 0;
      while (fs != ifs_code_tab[fs_code]) {fs_code++;}
//      t1div = round(hclk / fs);
//      fs = hclk / t1div;
      fs *= 1e3;
      ofs = (lb_base - log2(fs));
      oct_ofs = floor(ofs);
      note_ofs = round(1536.0 * (ofs - oct_ofs));
      printf("   {%5d,  %5d, %7d, %7d, %7d, %8d,  \"%c%c\" }", 
                poly_tab[j][i], fs_code, t1ms, t4ms, oct_ofs, note_ofs,
                ((t1ms / 10) ? '0'+(t1ms / 10) : ' '), 
                '0'+(t1ms % 10));
      if (sizeof(fs_tab)-1 > (i * sizeof(double))) printf(","); else printf(" ");
      printf("// fs=%6.3f kHz\r\xa", fs * 1e-3); 
    } // for (i = 
    printf(" }");
    if (0 == j) {
      printf(",\r\xa");
    }
    else {
      printf("\r\xa}; // const sys_config_st_t sys_config[]\r\xa");
    }
  } // for (j
  return(0);
} // int main()
