/* sbi2vma.c -- convert ALSA 2-OP FM voce file ".sb" file,         */
/*              SoundBlater ".BNK" file to YAMAHA MA-2 ".vma" file */

#pragma pack(1)

#include <stdio.h>
#include <string.h>

#define MAXVOICE 128
#define NAMELEN 16
#define DATALEN 26

// 'FM  '
#define FM_chunk_ID 0x20204d46

// 'SBI',0x1a
#define SBI_fourCC 0x1a494253

// 'OP2',0x1a
#define OP2_fourCC 0x1a32504F

// 'BNK',0x1a
#define BNK_fourCC 0x1a4b4e42

typedef struct sbi_header
{
  char key[4];
  char name[32];
}
sbi_header_t;

typedef struct opl3_AVEKM {
  unsigned char MULT: 4; // freq MULTiplier
  unsigned char KSR : 1; // Key Scaling for Rate
  unsigned char EGT : 1; // EG Type flag
  unsigned char VIB : 1; // VIBrato flag
  unsigned char AM  : 1; // Amplitude Modulation flag
} opl3_AVEKM_t;

typedef struct opl3_KSLTL {
  unsigned char TL  : 6; // Total   Level
  unsigned char KSL : 2; // Key Scaling for Level
} opl3_KSLTL_t;

typedef struct opl3_ARDR {
  unsigned char DR  : 4; // Decay   Rate
  unsigned char AR  : 4; // Attack  Rate
} opl3_ARDR_t;

typedef struct opl3_SLRR {
  unsigned char RR  : 4; // Release Rate
  unsigned char SL  : 4; // Sustain Level
} opl3_SLRR_t;

typedef struct opl3_WS {
  unsigned char WS  : 3; // Waveform Select
  unsigned char pad0: 5; // unused
} opl3_WS_t;

typedef struct opl3_FBC {
  unsigned char CON : 1; // Connection (algorithm)
  unsigned char FB  : 3; // FeedBack level
  unsigned char pad1: 4; // unused
} opl3_FBC_t;

typedef struct opl3_reg {
  opl3_AVEKM_t r0[2];
  opl3_KSLTL_t r1[2];
  opl3_ARDR_t  r2[2];
  opl3_SLRR_t  r3[2];
  opl3_WS_t    r4[2];
  opl3_FBC_t   r5[2];
} opl3_reg_t;

#define DATA_LEN_2OP    16
#define DATA_LEN_4OP    24

typedef struct sbi_inst
{
  sbi_header_t header;
  opl3_reg_t reg;
  char data[DATA_LEN_2OP - sizeof(opl3_reg_t)];
}
sbi_inst_t;

typedef struct ma2reg {
  unsigned char KSR : 1; // Key Scaling for Rate
  unsigned char SUS : 1; // SUStain flag
  unsigned char EGT : 1; // EG Type flag
  unsigned char VIB : 1; // VIBrato flag
  unsigned char MULT: 4; // freq MULTiplier
  unsigned char DR  : 4; // Decay   Rate
  unsigned char RR  : 4; // Release Rate
  unsigned char SL  : 4; // Sustain Level
  unsigned char AR  : 4; // Attack  Rate
  unsigned char KSL : 2; // Key Scaling for Level
  unsigned char TL  : 6; // Total   Level
  unsigned char WS  : 3; // Waveform Select
  unsigned char AM  : 1; // Amplitude Modulation flag
  unsigned char DAM : 2; // Depth of Amplitude Modulation
  unsigned char DVB : 2; // Depth of ViBrato
} ma2reg_t; // typedef struct ma2reg

typedef struct voicedata {
  unsigned char pad0;    // unused
  unsigned char bank;    // bank LSB
  unsigned char pcno;    // program change number
  unsigned char ALG : 3; // ALGorithm
  unsigned char FB  : 3; // FeedBack level
  unsigned char LFO : 2; // Low Frequency Osccillator
  unsigned char pad1;    // unused
  ma2reg_t op[4];        // MA2 operator data
  unsigned char pad2;    // unused 
} voicedata_t; // typedef struct voicedata

unsigned char vbuf[DATALEN];
long int nvoice;

sbi_inst_t sbi_inst;
int min_TL;
char instname[MAXVOICE][NAMELEN];
voicedata_t vdata[MAXVOICE];

int main()
{
  int i,j;
  unsigned long int f_id;
  char dum[4];
  unsigned char b;
  unsigned int bank, pc;

  nvoice = 0;
  min_TL = 15;

  memset(instname, 0, sizeof(instname));
  memset(vdata,    0, sizeof(vdata));

fprintf(stderr, "opl3_reg_t = %d\n", sizeof(opl3_reg_t));
  fread(&f_id, 4, 1, stdin); // read file ID
  if (BNK_fourCC == f_id) { // SoundBlaster BNK file
//    printf("BNK file.\n");
  } 
  else if ((SBI_fourCC == f_id) | (OP2_fourCC == f_id)) {
//    printf("SBI file.\n");
    
    while (0 != fread((&sbi_inst.header.name[0]), sizeof(sbi_inst_t) - 4, 1, stdin)) {
      voicedata_t *vdat_p = &(vdata[nvoice]);
      memcpy(instname[nvoice], sbi_inst.header.name, NAMELEN-1);
//      printf("%3d:%s ", nvoice+1, sbi_inst.header.name);
      vdat_p->pcno = nvoice; // instrument number
      vdat_p->FB   = sbi_inst.reg.r5[0].FB; // FeedBack level
      vdat_p->ALG  = sbi_inst.reg.r5[0].CON;// CONnection (ALGorithm)
      vdat_p->LFO  = 1; // LFO = 1 : 4.2 Hz, LFO = 2 : 6.1 Hz
      vdat_p->pad1 = 1; // unknown
      for (i = 0; i < 2; i++) { // for each operator
        opl3_reg_t *sbi_p = &(sbi_inst.reg);
        ma2reg_t   *ma2_p = &(vdat_p->op[i]);

//printf("%d: ", i);
        ma2_p->AM   = sbi_p->r0[i].AM;   // Amplitude Modulation
        ma2_p->VIB  = sbi_p->r0[i].VIB;  // VIBrato
        ma2_p->EGT  = sbi_p->r0[i].EGT;  // EG Type
        ma2_p->KSR  = sbi_p->r0[i].KSR;  // Key Scaling Rate
        ma2_p->MULT = sbi_p->r0[i].MULT; // frequency MULTiplier
        ma2_p->KSL  = sbi_p->r1[i].KSL;  // Key Scaling Level
        ma2_p->TL   = sbi_p->r1[i].TL;   // Total Level
        if ((1 == i) & (!(vdat_p->ALG))) {  // OP1 of ALG=0
          if (min_TL > ma2_p->TL) min_TL = ma2_p->TL; // record minimum TL
        } // if
//fprintf(stderr,"%3d: OP%d TL=%d\n", nvoice, i, ma2_p->TL);
        ma2_p->AR   = sbi_p->r2[i].AR;   // Attack Rate
        ma2_p->DR   = sbi_p->r2[i].DR;   // Decay  Rate
        ma2_p->SL   = sbi_p->r3[i].SL;   // Sustain Level
        ma2_p->RR   = sbi_p->r3[i].RR;   // Release Rate
        ma2_p->WS   = sbi_p->r4[i].WS;   // Wave Select
//if (ma2_p->VIB) printf("V ");
//if (ma2_p->AM)  printf("A ");
      } // for (i = 0; 
//      printf("\r\xa");
      nvoice++;
      fread(&f_id, 4, 1, stdin); // read file ID
    } // while
fprintf(stderr,"minimum TL = %d\r\xa", min_TL);
    j = FM_chunk_ID;
    fwrite(&j, 4, 1, stdout); // 'FM  ' chunk ID
    j = nvoice * (DATALEN + NAMELEN); // calculate chunk size
    for (i = 0; i < 4; i++) {
      printf("%c", 0xff & (j >> 24));
      j <<= 8; 
    } // for (i 
    for (i = 0; i < nvoice; i++) { // instrument name
      fwrite(instname[i], NAMELEN, 1, stdout);
    } // for (i
    for (i = 0; i < nvoice; i++) { // voice data
      if (!(vdata[i].ALG)) { // series algorithm
        vdata[i].op[1].TL -= min_TL; // adjust output level
      }
      fwrite(&(vdata[i]), DATALEN, 1, stdout);
    } // for (i
  } 
  else fprintf(stderr, "(sbi2vma)Error: unsupported file type.\n");
  return(0);
} // int main()
