;
; lcd4uart:  4-bit mode character LCD and keySW I/F
;            using UART (62.5 kbps) to HOST
;  
; 2011-Apr-19, created by pcm1723
;
; (PIC16F628)
;
; CLOCK FREQUENCY: 4 MHz (internal RC osc)
;
; INPUT SIGNAL:
; 
;  RB1/RX (Pin 7) -- connect to HOST TxD
;
; OUTPUT SIGNAL:
; 
;  RA3 (Pin  2) -- connect to LCD DB7
;  RA2 (Pin  1) -- connect to LCD DB6
;  RA1 (Pin 18) -- connect to LCD DB5
;  RA0 (Pin 17) -- connect to LCD DB4
;
;  RA4 (Pin  3) -- connect to LCD RS
;
;  RB0 (Pin  7) -- connect to LCD E
;
;  RB2/TX (Pin 8) -- connect to HOST RxD
;
; Required peripherals: (PIC16F628)
;
;   USART module
;
; Function:
;
;
        processor pic16f628
;
        List    p=pic16f628,f=inhx8m
        include "p16f628.inc"
        __config _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_OFF
;
        udata   0020h
rxdata  res     1            ; Rx data buffer
txdata  res     1            ; Tx data buffer
;
; 
        code    0000h
        goto    start        ; reset vector
        res     1
        res     1
        res     1
        goto    start        ; interrupt vector
;
start:
        call    port_setup   ; setup for CCP1 out & oct code in
        call    usart_setup  ; setup for USART and BRG
        banksel PIR1         ; PIR1, RCREG, PORTA,
                             ; PORTB, TXREG on BANK0
rxloop0:
        btfss   PIR1,RCIF    ; skip if RCIF=1 (RCREG full)
        goto    rxloop0      ; loop if RCIF=0 (RCREG empty)
        movfw   RCREG        ; read the received data
        movwf   PORTA        ; output the data to LCD RS, D7..D4
        bsf     PORTB,0      ; output '1' to LCD_E (RB0)
        swapf   PORTB,w      ; PORTB[7:4] --> w[3:0]
        iorlw   b'11110000'  ; fill "1" in w[7:4]
        movwf   TXREG        ; output to USART
txloop1:
        banksel TXSTA        ; TXSTA on BANK1
        btfss   TXSTA,TRMT   ; skip if TRMT=1 (TSR empty)
        goto    txloop1      ; loop if TRMT=0 (TSR full)
        banksel PORTB        ; PORTB, PIR1 on BANK0
        bcf     PORTB,0      ; output '0' to LCD_E (RB0)

        goto    rxloop0
;
;
; port setup for LCD interface
;
port_setup:
        banksel TRISA        ; TRISA, TRISB, OPTION on BANK1
        bcf     TRISA,4      ; RA4 output enable (LCD RS)
        bcf     TRISA,3      ; RA3 output enable (LCD DB7)
        bcf     TRISA,2      ; RA2 output enable (LCD DB6)
        bcf     TRISA,1      ; RA1 output enable (LCD DB5)
        bcf     TRISA,0      ; RA0 output enable (LCD DB4)
        bcf     TRISB,0      ; RB0 output enable (LCD E)
        bcf     TRISB,2      ; RB2/TX output enable
        bcf     OPTION_REG,NOT_RBPU ; enable PORTB weak pull up
        banksel PORTB        ; PORTB on BANK0
        bcf     PORTB,0      ; RB0 = '0' (LCD E)
        return
;
; setup for USART and Baud Rate Generator for 62.5 kbps at 4 MHz osc
;
usart_setup:
        movlw   b'00100000'  ; TX enable, async, BRGH=0 (low speed)
        banksel TXSTA        ; TXSTA, SPBRG on BANK1
        movwf   TXSTA        ; set Tx config
        movlw   d'0'         ; Baud rate = 4 [MHz] / 64 / (0+1) = 62.5 [kbps]
        movwf   SPBRG        ; set Baud rate
        movlw   b'10010000'  ; SPEN=1, CREN=1
        banksel RCSTA        ; RCSTA on BANK0
        movwf   RCSTA        ; set Rx config
        return
;       
        end
