/****************************************************/
/*                                                  */
/* TGRX62N.h : RX62N tone generator header file     */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/29 Rev. 1.2 suport stereo, SMF sequencer */
/* 2011/04/24 Rev. 1.1 by pcm1723                   */
/*                                                  */
/****************************************************/
#ifndef RX62N_H
#define RX62N_H

#if defined(__RX)
//
// compile time global options
//

// define "MIDIONLY" for disable DEMO playback
//
// #define MIDIONLY
// #define DEMOONLY

// define   "USE_SMF" for SMF  sequencer (V1.2 new feature)
// undefine "USE_SMF" for SMAF sequencer (V1.1 or older compatible)
#define USE_SMF 1

#define MAXSEQCH      32
#define ABS_MAX_UNITS 32
#define ABS_MAX_PARTS 32

//
// STEREO_MODE = 1 for stereo
// STEREO_MODE = 0 for monaural
#define STEREO_MODE 1

#define USEWAVEQUE
#define WAVEQUESIZE 256

#define debug 0

#define REVERB 1

// #define FORCE_LEGACY_MIDI
// #define FORCE_SERIAL_MIDI

//#define FORCE_LOW

//
// LCD related
//
#define LCD
#define LCD_4bit 1
#define BANNER_STRING "FRK-RX62N   MIDIFM ToneGen V1.2 "

//
// demo / sequencer related
//
#ifndef MIDIONLY
  #define SEQUENCER
#endif

#define EXTRADEMO 1

//
// machine dependent defines
//

// DAC input format selection
// 1 for "Standard" format  (16-bit, right justify, LRCK: 1=L, 0=R)
// 0 for BU9480 only format (16-bit, right justify, LRCK: 1=L, 0=R, Lch data only)
  #define DAC_FORMAT_SEL 0
//
// 10 bit A/D converter for CPU load measurement
// use AD1
//
  #define LOAD_ADC        AD1
  #define MSTP_LOAD_ADC   MSTP_AD1
  #define LOAD_ADC_PORT   PORT4
//  
// measurement input is AN5/P45 (CN2-6)
// result in AD1.ADDRB
//
  #define LOAD_ADC_BITPOS B5
  #define LOAD_ADC_CH     1
  #define LOAD_ADC_BIT(x) LOAD_ADC_PORT.x.BIT.LOAD_ADC_BITPOS
//
// macro for ADC READ and align to 12 bit boundary
//
  #define READ_ADC  (LOAD_ADC.ADDRB >> 4)
//
// macro for ADC START
//
  #define START_ADC (LOAD_ADC.ADCSR.BIT.ADST = 1)
//
// SCI define for MIDI I/O
// use SCI0, P20/TxD0 (CN2-20), P21/RxD0 (CN2-21)
// use P22 (CN2-22) as general input port for MIDI Baudrate sens
//
  #define MIDI_SCI             SCI0
  #define MSTP_MIDI_SCI        MSTP_SCI0
  #define MIDI_SCI_Rx_PORT     PORT2
  #define MIDI_SCI_Tx_PORT     PORT2
  #define MIDI_SCI_SENS_PORT   PORT2
  #define MIDI_SCI_Rx_BITPOS   B1
  #define MIDI_SCI_Tx_BITPOS   B0
  #define MIDI_SCI_SENS_BITPOS B2
//  
  #define MIDI_SCI_Rx_BIT(x)   MIDI_SCI_Rx_PORT.x.BIT.MIDI_SCI_Rx_BITPOS
  #define MIDI_SCI_Tx_BIT(x)   MIDI_SCI_Tx_PORT.x.BIT.MIDI_SCI_Tx_BITPOS
  #define MIDI_SCI_SENS_BIT(x) MIDI_SCI_SENS_PORT.x.BIT.MIDI_SCI_SENS_BITPOS
//
// SCI define for character LCD interface
// use SCI6, P00/TxD6-A (CN2-29), P01/RxD6-A (CN2-30)
//
  #define LCD_SCI           SCI6
  #define MSTP_LCD_SCI      MSTP_SCI6  
  #define LCD_SCI_Rx_PORT   PORT0
  #define LCD_SCI_Tx_PORT   PORT0
  #define LCD_SCI_Rx_BITPOS B1
  #define LCD_SCI_Tx_BITPOS B0
//  
  #define LCD_SCI_Rx_BIT(x) LCD_SCI_Rx_PORT.x.BIT.LCD_SCI_Rx_BITPOS
  #define LCD_SCI_Tx_BIT(x) LCD_SCI_Tx_PORT.x.BIT.LCD_SCI_Tx_BITPOS
//  
// SCIn PIN FUNCTION REGISTER define  
//
  #define LCD_SCI_PFF       IOPORT.PFFSCI.BIT.SCI6S
// 0 for '-A' pins, 1 for '-B' pins  
  #define LCD_SCI_SEL_AB    (0)
//
// LCD / SW define
//
  #define LCD_RSBIT         (0x10)
  #define SW_RE_BITMASK     0x03
  #define SW_RE_DETENT      0x03
  #define SW_RE_FWD         0x02
  #define SW_RE_BACK        0x01
//
// RSPI define for serial DAC
// use RSPI1
// 
  #define SDAC_SPI      RSPI1
  #define MSTP_SDAC_SPI MSTP_RSPI1 
//  
// RSPIn PIN FUNCTION REGISTER define  
//
  #define SDAC_SPI_PFREG(x) IOPORT.PFHSPI.x
// 0 for '-A' pins, 1 for '-B' pins  
  #define SDAC_SPI_ABSEL (0) 
// LED
  #define LED_OFF (PORT1.DR.BIT.B5 = 1)
  #define LED_ON  (PORT1.DR.BIT.B5 = 0)
  #ifdef DEMOONLY
    #define CHECKMIDI (0x00)
  #else
    #define CHECKMIDI (MIDI_SCI.SSR.BIT.RDRF)
  #endif // #ifdef DEMOONLY 
//  
// MIDI I/O macro
//
  #define GETMIDI        (MIDI_SCI.RDR)
  #define PUTMIDI(x)     (MIDI_SCI.TDR = x)
  #define MIDI_OVRUN     (MIDI_SCI.SSR.BIT.ORER)
  #define CLR_MIDI_OVRUN (MIDI_SCI.SSR.BYTE = 0xC0)
//
// LCD / SW input macro  
//
  #define CHECK_KEYSW     (LCD_SCI.SSR.BIT.RDRF)
  #define KEYSW_OVRUN     (LCD_SCI.SSR.BIT.ORER)
  #define CLR_KEYSW_OVRUN (LCD_SCI.SSR.BYTE = 0xC0)
  #define GET_KEYSW       (LCD_SCI.RDR)
  #define PUT_LCD(x)      (LCD_SCI.TDR = (0xE0 | (x)))
  #define GET_PANEL_KEY   (PORT3.PORT.BIT.B5) // NMI/P35
//
// digital delay definition
//  
  #define DEFAULT_DELAY_ms 120
  #define DELAYMEMSIZE (48*DEFAULT_DELAY_ms)
//  
  extern void RX62N_port_setup( void );
  
  #define MAX_CONFIG 5

// #define JITTER_TEST
// #define USE_DMA
#endif // #if defined(__RX)
#endif // #ifndef RX62N_H