/****************************************************/
/*                                                  */
/* hw_RX62N.c : RX62N specific routines             */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/29 Rev. 1.2 support stereo (CC#10 PAN)   */
/* 2011/04/24 Rev. 1.1 by pcm1723                   */
/*                                                  */
/****************************************************/


#include "iodefine.h"
#include "vect.h"
#include <machine.h>
#include "../common/tonegen.h"

volatile int16_t DA_buf  = 0;
volatile int16_t DA_buf2 = 0;

// 
// set configuration values for specified mode 
//
void set_config(int mode)
{
//  MTUA.TSTR.BIT.CST0  = 0; // stop timer 0
  MTU0.TGRD  = sys_config[fastchip][mode].t1div - 1; // set Timer divisor constant
  MTU0.TGRC  = (sys_config[fastchip][mode].t1div/2) - 1; // PWM duty = 50 %
  MAXUNITS   = sys_config[fastchip][mode].polyphony; // polyphony = number of units
  SEQTICK1ms = sys_config[fastchip][mode].tick1ms;   // tick count for 1ms
  SEQTICK4ms = sys_config[fastchip][mode].tick4ms;   // tick count for 4ms
  oct_ofs    = sys_config[fastchip][mode].oct_ofs;   // octave offset
  note_ofs   = sys_config[fastchip][mode].note_ofs;  // f-number table offset
  memcpy(&FS_ASC, sys_config[fastchip][mode].fs_asc, 2);    // ASCII code for fs

#ifdef REVERB
  delay_mem_size = SEQTICK1ms * delay_ms; // calc delay momory size
  if (DELAYMEMSIZE < delay_mem_size) delay_mem_size = DELAYMEMSIZE; // clip if too big
  delayp = 0;
#endif
  midi_mon_part = 0;  // reset MIDI monitor part
  MAXMIDICH  = MAXUNITS; // supported MIDI channels
  
//  MTUA.TSTR.BIT.CST0  = 1; // start timer 0
} // void set_config()

void enable_interrupt(void)
{
//    set_imask(0); // in <machine.h>
} // void enable_interrupt() 

//
// Exception occurs at rising edge of LRCK (TIOC0A)
// for L-ch audio data output timing
//
void Excep_MTU0_TGIA0(void)
{
  t1_tick = 1;
#if (STEREO_MODE)
  DA_buf2 = wave_que[wq_rd_ix][0];  // get L-ch wave data
  DA_buf  = wave_que[wq_rd_ix][1];  // get R-ch wave data
	SDAC_SPI.SPDR.WORD.H = DA_buf2;   // L-ch data
#else // Monaural mode  
  DA_buf = wave_que[wq_rd_ix][0];  // get wave data
	SDAC_SPI.SPDR.WORD.H = DA_buf;   // L-ch data
#endif  
#if defined(USE_DMA)
  DA_buf2 = (0x8000 ^ DA_buf);  // convert 2'C to offset binary
#else  
  #if (STEREO_MODE)
  *((uint32_t *)&DA.DADR0) = (0x80008000UL ^ (((int32_t)DA_buf << 16) | (uint16_t)DA_buf2)); // convert 2'C to offset binary
  #else // Monaural mode
  DA.DADR0 = (0x8000 ^ DA_buf); // convert 2'C to offset binary
  #endif
#endif
  wq_rd_ix = (wq_rd_ix + 1) & (WAVEQUESIZE-1); // advance index
} // void Excep_MTU_TGIA0()

//
// Exception occurs at falling edge of LRCK (TIOC0A)
// for R-ch audio data output timing
//
void Excep_MTU0_TGIB0(void)
{
	SDAC_SPI.SPDR.WORD.H = DA_buf;
} // void Excep_MTU_TGIB0()

void GPIO_setup( void )
{
// LED port setup    
  PORT1.DDR.BIT.B5 = 1; // P15 = output
  PORT1.DR.BIT.B5  = 0; // LED ON
#if   defined(FORCE_SERIAL_MIDI)
  serial_midi = 1; // force serial MIDI
#elif defined(FORCE_LEGACY_MIDI)
  serial_midi = 0; // force (legacy) MIDI
#else
// serial MIDI sens port setup
	MIDI_SCI_SENS_BIT( DDR ) = 0; // SENS bit mode = GPIO input
	serial_midi = MIDI_SCI_SENS_BIT( PORT ); // read SENS bit
#endif // #if defined()...
} // void GPIO_setup()

//
// SCI (Serial Communication Interface) setup for MIDI I/O
//
void SCI_setup( void )
{
  MSTP( MIDI_SCI ) = 0; // enable SCIn for MIDI I/O
    
  MIDI_SCI.SMR.BYTE = 0x00; // PCLK/1, 8N1 ASYNC mode
  MIDI_SCI.SCR.BYTE = 0x00; // use built-in Baudrate gen, disable Tx, Rx
  MIDI_SCI.SEMR.BIT.ABCS = 0; // select BPS x16 sampling
  if (serial_midi) { // set Baud rate to 38.4 kbps (serial MIDI)
    MIDI_SCI.BRR = 38;     // 48 MHz/(38+1)/32=38.462 kbps (+0.16%)
  } else { // set Baud rate to 31.25 kbps (legacy MIDI)
    MIDI_SCI.BRR = 47;     // 48 MHz/(47+1)/32=31.25 kbps (+-0%)
  } // if (serial_midi) ...
  MIDI_SCI_Rx_BIT( ICR ) = 1; // enable input buffer 
  MIDI_SCI.SCR.BYTE  |= 0x30; // enable Tx, Rx
#if defined(LCD)
//
// SCIn setup for LCD interface
//
  MSTP( LCD_SCI ) = 0; // enable SCI6 clock
  
  LCD_SCI_PFF = LCD_SCI_SEL_AB; // select SCIn-A / SCIn-B
  
  LCD_SCI.SCR.BYTE = 0x00;   // use built-in Baudrate gen, disable Tx, Rx
  LCD_SCI.SMR.BYTE = 0x00;   // PCLK/1, 8N1 ASYNC mode
  LCD_SCI.SEMR.BIT.ABCS = 0; // select BPS x16 sampling
  LCD_SCI.BRR = (24 - 1);    // 48 MHz/24/32 = 62.5 kbps
  LCD_SCI_Rx_BIT( ICR ) = 1; // enable input buffer 
  LCD_SCI_Rx_BIT( ODR ) = 1; // enable open drain 
  IOPORT.PFBTMR.BIT.TMR0S = 1; // select TMR0 input on P20 / P21
  IOPORT.PF8IRQ.BIT.ITS9  = 1; // select IRQ9 input on P41
  LCD_SCI.SCR.BYTE  = 0x30; // enable Tx, Rx
#endif    
    
} // void SCI_setup()

//
// MTU setup for LRCK generation
//
void MTU_setup(int Lch_int, int Rch_int)
{
#define INIT_T1DIV (1000)
    MSTP( MTU0 ) = 0; // enable MTU0
    MTU0.TCR.BIT.CCLR   = 2; // clear TCNT on TGRB compare match
    MTU0.TCR.BIT.CKEG   = 0; // count on rising edge
    MTU0.TCR.BIT.TPSC   = 0; // clock = P_phi / 1
//    MTUA.TOER.BIT.OE4C = 1; // enable TIOC0C outut
    MTU0.TMDR.BIT.BFA   = 1; // buffered TGRA_0/TGRC_0
    MTU0.TMDR.BIT.BFB   = 1; // buffered TGRB_0/TGRD_0
    MTU0.TMDR.BIT.MD    = 3; // PWM mode 2
    MTU0.TBTM.BIT.TTSB  = 1; // TGRD -> TGRB at TCNT clear
//    MTU0.TIORL.BIT.IOC = 3; // TIOC0C toggle on TGRC_0 compare match
    MTU0.TIORH.BIT.IOA  = 2; // TIOC0A High at TGRA_0 compare match
    MTU0.TGRC           = (INIT_T1DIV >> 1) - 1;// PWM duty = 50 %
    MTU0.TGRD           = INIT_T1DIV - 1;     // fs = 48 [MHz] / 1000 = 48 [kHz]
	if (Lch_int) {             // L-ch interrupt enabled
      MTU0.TIER.BIT.TGIEA = 1; // enable TGRA interrupt
      IEN( MTU0, TGIA0)   = 1; // interrupt enable
      IPR( MTU0, TGIA0)   = Lch_int; // interrupt priority
	} else {                   // L-ch interrupt disabled
      MTU0.TIER.BIT.TGIEA = 0; // disable TGRA interrupt
	} // if (Lch_int)
	if (Rch_int) {             // R-ch interrupt enabled
      MTU0.TIER.BIT.TGIEB = 1; // enable TGRB interrupt
      IEN( MTU0, TGIB0)   = 1; // interrupt enable
      IPR( MTU0, TGIB0)   = Rch_int; // interrupt priority
	} else {                   // L-ch interrupt disabled
      MTU0.TIER.BIT.TGIEB = 0; // disable TGRB interrupt
	} // if (Rch_int)
    MTUA.TSTR.BIT.CST0  = 1; // start timer 0
} // void MTU_setup()

void DAC_setup(int dac0_en, int dac1_en)
{
  MSTP( DA ) = 0;          // enable DAC module
  DA.DADPR.BIT.DPSEL  = 1; // MSB left justified
  if (dac0_en) {           // DAC0 enable
    DA.DACR.BIT.DAOE0 = 1; // enable DAC0
    PORT0.DDR.BIT.B3  = 0; // force input mode on P03/DA0
    PORT0.ICR.BIT.B3  = 0; // force input mode on P03/DA0
  } // if (dac0_en) ...
  if (dac1_en) {           // DAC1 enable
    DA.DACR.BIT.DAOE1 = 1; // enable DAC1
    PORT0.DDR.BIT.B5  = 0; // force input mode on P05/DA1
    PORT0.ICR.BIT.B5  = 0; // force input mode on P05/DA1
  } // if (dac0_en) ...
} // void DAC_setup()

// 
// RSPI setup for serial digital audio stereo DAC
//
void SPI_setup( void )
{
  MSTP( SDAC_SPI ) = 0; // enable RSPIn clock
  SDAC_SPI_PFREG( BIT.RSPIS  ) = SDAC_SPI_ABSEL; // select RSPIn-A/-B signals
  SDAC_SPI_PFREG( BIT.RSPCKE ) = 1; // enable RSPCKn pin
  SDAC_SPI_PFREG( BIT.MOSIE  ) = 1; // enable MOSIn pin
  
  SDAC_SPI.SPCR.BIT.SPE    = 0; // disable SPI for mode setup
  SDAC_SPI.SPCR.BIT.SPMS   = 1; // 3-wire mode
  SDAC_SPI.SPCR.BIT.TXMD   = 1; // transmit only
  SDAC_SPI.SPCR.BIT.MSTR   = 1; // master mode
  SDAC_SPI.SPCR.BIT.MODFEN = 0; // disable MODe Fault error detection
  
  SDAC_SPI.SPPCR.BIT.SPLP  = 0; // no loopback
  SDAC_SPI.SPDCR.BIT.SPLW  = 0; // 16 bit data access
  
  SDAC_SPI.SPBR.BYTE       = (24 - 1) / 2; // RSPCK = 48 [MHz] / 24 = 2 [MHz]
  
  SDAC_SPI.SPCMD0.BIT.SCKDEN = 0; // disable RSPCK delay
  SDAC_SPI.SPCMD0.BIT.SLNDEN = 0; // SSL negate delay
  SDAC_SPI.SPCMD0.BIT.SPNDEN = 0; // next access delay
  SDAC_SPI.SPCMD0.BIT.SSLKP  = 0; // SSL keep
  SDAC_SPI.SPCMD0.BIT.LSBF   = 0; // MSB First
  SDAC_SPI.SPCMD0.BIT.SPB    = 0xf; // 16 bit transfer
  SDAC_SPI.SPCMD0.BIT.BRDV   = 0; // base Baudrate
  SDAC_SPI.SPCMD0.BIT.CPHA   = 0; // Clock PHAse = 0 (RSPCK pos edge sample)
  SDAC_SPI.SPCMD0.BIT.CPOL   = 0; // Clock POLarity = 0 (RSPCK idle = LOW)
  
  SDAC_SPI.SPCR.BIT.SPE      = 1; // enable SPI
//  SDAC_SPI.SPDR.LONG       = 0x0;
} // void SPI_setup()

void CPG_setup()
{
// ICLK = 96 MHz, BCLK = 24 MHz, PCLK = 48 MHz	
  SYSTEM.SCKCR.LONG = 0x00020100;
} // void CPG_setup()

//
// ADC setup for CPU load measurement
//
void ADC_setup( void )
{
  MSTP( LOAD_ADC )         = 0;   // enable 10 bit ADn clocking
  
  LOAD_ADC_BIT( DDR )      = 0;   // set ANx as input port
  LOAD_ADC_BIT( ICR )      = 0;   // disable digital input buffer on ANx
  
  LOAD_ADC.ADCSR.BYTE      = 0;   // disable ADn
  LOAD_ADC.ADCSR.BIT.CH    = LOAD_ADC_CH; // set input MUX ch
  LOAD_ADC.ADCR.BYTE       = 0;   // single mode, PCLK/8, soft-trig
  LOAD_ADC.ADDPR.BIT.DPSEL = 1;   // MSB left justify
  LOAD_ADC.ADSSTR          = 255; // sampling states = 255 (maximum)
} // void ADC_setup()


void RX62N_port_setup( void )
{   
  CPG_setup();     // Clock Pulse Generator setup
  GPIO_setup();    // General Purpose I/O setup
	SPI_setup();    // SPI setup for serial DAC I/F
  SCI_setup();     // SCI setup for MIDI I/O
	DAC_setup(1, STEREO_MODE); // enable DAC0 / DAC1
  ADC_setup();     // A/D converter setup
  MTU_setup(1, (STEREO_MODE | DAC_FORMAT_SEL)); // MTU setup for LRCK generation
} // void RX62N_port_setup()
