/****************************************************/
/*                                                  */
/* lcd.h : LCD and SW sens define                   */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/20 Rev. 1.01 by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef LCD_H
#define LCD_H

#if defined(LCD)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//
// macros
//
#define LCD_ROWS  2
#define LCD_COLS 16
#define LCD_BUFSIZE (LCD_ROWS*LCD_COLS)
#define CGRAMSIZE 64
// time constant for 3 sec in terms of 4 ms tick
#define LCD_DISP_TICKS 750
#define LCD_BLINK_ms   500
#define LCD_BLINK_CHAR 0xa5
#define MON_CH_DISP_OFS  (0)
#define MON_SW_DISP_OFS  (1)
#define MON_PC_DISP_OFS  (MON_CH_DISP_OFS+6)
#define MON_CC_DISP_OFS  (MON_CH_DISP_OFS+11)

#define MON_KEY_DISP_OFS (LCD_COLS+2)
#define MON_OCT_DISP_OFS (2)
#define MON_NN_DISP_OFS  (MON_KEY_DISP_OFS-2)
#define MON_VEL_DISP_OFS  (MON_KEY_DISP_OFS+12)

#define MON_RPN_DISP_OFS (6)
#define MON_RPN_MSB_OFS  (11)

#define SW_INC_BIT      0x01
#define SW_DEC_BIT      0x02
#define SW_SFT_BIT      0x04

#define GET_INC_KEY (SW_INC_BIT & sw_bits)
#define GET_DEC_KEY (SW_DEC_BIT & sw_bits)
#define GET_SFT_KEY (SW_SFT_BIT & sw_bits)

#define DEBOUNCE_ms (14)

// PRESS/CLICK decision threshold (29 * 14 ms = 406 ms) 
#define SW_RL_TH (29)

enum panel_mode_t {NORMAL, MIDI_MON, CH_ASSIGN, SHOW_NRPN, SHOW_Sys_Ex, SET_CONFIG, SYS_RESET};
enum sw_value_t {NOCHANGE, CLICK, PRESS};

// 
// variables
//
extern char banner_str[] ;
extern const char hex_str[17];
extern const uint8_t cgram[CGRAMSIZE];
extern uint8_t  lcd_buf[LCD_BUFSIZE];
extern uint8_t  lcd_buf2[LCD_BUFSIZE];
extern uint8_t  *lcd_buf_p;
extern uint8_t  *lcd_buf_w;
extern uint8_t  lcd_row, lcd_col, lcd_buf_ind;
extern uint8_t  lcd_disp_row, lcd_disp_col, lcd_cur_pos;
extern int16_t  lcd_blink_timer;
extern uint8_t  lcd_blink_flag;
extern int16_t  lcd_disp_timer;
extern uint8_t  sw_bits;
extern uint32_t inc_sw_q, dec_sw_q;
extern uint16_t inc_sw_rl, dec_sw_rl;
extern int8_t   sw_val;
extern int8_t   re_state;

//
// functions
//
extern void lcd_puthex1(int i);
extern void lcd_puthex2(int i);
extern int  lcd_putchar(int c);
extern void lcd_putstr(char* s);
extern void lcd_draw_panel(int panel_no);
extern void lcd_write4(int d);
extern void lcd_write8(int d);
extern void lcd_refresh(uint8_t s);
extern void lcd_init();
extern void lcd_clear(uint8_t* lcd_buf);
extern void lcd_gotoxy(int x, int y);
extern void wait_ms(int ms);
extern void out_LCD(int d, int s);
extern void lcd_cpuload(void);
extern void lcd_E_down(void);
extern void lcd_dbg_hex1(uint8_t pos, int8_t n);
extern void lcd_dbg_hex2(uint8_t pos, int16_t n);
extern void lcd_dbg_hex4(uint8_t pos, int16_t n);
extern void lcd_dbg_hex8(uint8_t pos, int32_t n);
extern void lcd_lba_hex1(uint8_t pos, int8_t n);
extern void lcd_lba_hex2(uint8_t pos, int8_t n);
extern int8_t sw_input(void);
extern void panel_cmd(int8_t sw);
extern void print_banner(void);

#if defined(__v850__)
#define LCD_EBIT  0x40
#define LCD_RSBIT 0x10
#define LCD_DATA_PORT     P3L
#define LCD_DATA_DDR      PM3L
#define LCD_DATA_PIN      P3L
#define LCD_DATA_BITMASK  0x78
#define LCD_DATA_SFT      1

#define LCD_CTRL_PORT     P0
#define LCD_CTRL_DDR      PM0

#define ADC_DATA_PORT     ADA0CR11
#define ADC_BITPOS        0
#define ADC_BITMASK       0xff

#define SW_BITMASK        LCD_DATA_BITMASK
#define SW_BIT_SHIFT      3
#define SW_SENS_PORT      LCD_DATA_PORT
#define SW_SENS_DDR       LCD_DATA_DDR
#define SW_SENS_PIN       LCD_DATA_PIN
#define SW_SENS_BITMASK   LCD_DATA_BITMASK

#define SW_RE_BITMASK     0x03
#define SW_RE_DETENT      0x03
#define SW_RE_FWD         0x02
#define SW_RE_BACK        0x01

#define SMIDI_SENS_PORT    LCD_DATA_PIN
#define SMIDI_SENS_BITMASK 0x40

#define LCD_DATA_PORT_OUT(port,mask,x,s) (port = (~mask & port) | (mask & ((x)>>s)))

#define LCD_DATA_OUT(x) (LCD_DATA_PORT_OUT(LCD_DATA_PORT,LCD_DATA_BITMASK,x,LCD_DATA_SFT))

#define LCD_CTRL_OUT(e,rs) (LCD_CTRL_PORT = (~(LCD_EBIT | LCD_RSBIT) & LCD_CTRL_PORT) | (LCD_EBIT & (e)) | (LCD_RSBIT & (rs)))
#define LCD_E_OUT(e) (LCD_CTRL_PORT = (LCD_EBIT & e) | (~LCD_EBIT & LCD_CTRL_PORT))

#endif // #if defined(__V850__)

#endif // #if defined(LCD)

#endif // LCD_H