/*********************************************************/
/*                                                       */
/* mcu_lcd.c : Character LCD interface in 4-bit mode     */
/*             via 8-bit MCU (PIC, AVR etc.)             */
/*                                                       */
/* NON COMMERCIAL USE ONLY                               */
/*                                                       */
/* 2011/04/20 Rev. 1.1 by pcm1723                        */
/*                                                       */
/*********************************************************/

#include "../common/tonegen.h"

#if defined(LCD)

uint8_t sw_bits;

// negate LCD E signal and read ADC data 
void lcd_E_down(void)
{
  adc_data = READ_ADC; // read ADC data and align to 12 bit boundary
  START_ADC;  // start ADC 
} // void lcd_E_down()

void out_LCD(int d, int s)
{
  if (KEYSW_OVRUN) { // if overrun error occurred
    CLR_KEYSW_OVRUN; // clear overrun error to continue
  } // if
  if (CHECK_KEYSW) { // get key SW value
    sw_bits  = GET_KEYSW;
    sw_bits &= (0xfe | GET_PANEL_KEY); // combine with on-board SW
  } else {// if 
    sw_bits = (0x6 | GET_PANEL_KEY); // on-board SW only
  } // if (CHECK_KEYSW)
  d = (0xff & d) | (0xff00 & ( s ? d << 4 : d)); // low nibble if (1 == s)
  PUT_LCD((LCD_RSBIT & d) | (0x0f & (d >> 12))); // out high nibble
} // void out_LCD()

// display CPU load in 2 digit (max 99) 
void lcd_cpuload(void)
{
  int i, d;
  d = adc_data; // get ADC data in b11..b0, (0.0 <= d < 1.0) at Q12 format
  lcd_gotoxy(13, 0);  // set position
  for (i = 0; i < 2; i++) { // convert to decimal two digit
    d *= 10; // multiply by 10, (0.0 <= d < 10.0) at Q12 format
    lcd_putchar(hex_str[0x0f & (d >> 12)]); // extract b15..b12 and convert to ASCII
    d &= 0x0fff; // remove integer part of Q12 format
  } // for
} // void lcd_cpuload()

//
// wait specified milli-seconds by counting 1 ms tick flag
//
void wait_ms(int ms)
{
  int i, j;
  for (i = 0; i < ms; i++) {
    for (j = 0; j < SEQTICK1ms; j++) {
      while (0 == t1_tick) ;
      t1_tick = 0;
    } // for (j
  } // for (i
} // void wait_ms()

// write high nibble (4bit config) 
void lcd_write4(int d)
{
  PUT_LCD((LCD_RSBIT & d) | (0x0f & (d >> 12)));
  wait_ms(1);              // wait for 1ms
} // void lcd_write4()

// for 4bit configuration, write high nibble first, then write low nibble
void lcd_write8(int d)
{
  lcd_write4(d);      //  write full 8bit (8bit config), or write high nibble (4bit config)
#ifdef LCD_4bit
  lcd_write4((0xff & d) | ((0x0f00 & d) << 4)); //  write full 8bit (8bit config), or write high nibble (4bit config)
#endif
} // void lcd_write8()

#endif
