/****************************************************/
/*                                                  */
/* op8k.h : OPERATOR define with 8 K sine table     */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/20 Rev. 1.01 by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef OP_H
#define OP_H

/* Sine table size */
#define STAB_SIZE 8192

/* operator property definition */
typedef struct {
    uint32_t  phacc;   // phase accumulator 
    uint32_t  ph_inc;  // phase increment (pitch) 
    int32_t   mod_in;  // phase modulation input (modulator or feedback)
    uint32_t  egacc;   // EG accumulator           
    uint32_t  attacc;  // EG attack accumulator           
    int32_t   eginc;   // EG acc increment        
    int32_t   eginit;  // EG acc init value        
    int32_t   ol_lin;  // current out level in lin 
    int32_t   tl_lin;  // total level in linear    
    int32_t   sl_lin;  // sustail level in linear  
    int32_t   tl_lb;   // TL + KSL + VEL in log2      
    int32_t   sl_lb;   // sustain level in log2      
    int32_t   vol_lb;  // VOL + EXP in log2      
    int32_t   am_lb;   // AM in log2      
    int16_t   egrate;  // EG rate                  
    int16_t   opout;   // wave output of this op  
    op_vparam_t vp; // voice parameters
    uint8_t  egmode;  // EG mode 
    uint8_t  last_egm; // last EG mode                 
    uint8_t  rof;     // KSR offset (0..15)  
    int8_t   gate_q;  // key-ON gate queue
    int8_t   damp_q;  // damper queue
  } op_prop_t;

extern int16_t op_ws0(op_prop_t *o_p);
extern int16_t op_ws1(op_prop_t *o_p);
extern int16_t op_ws2(op_prop_t *o_p);
extern int16_t op_ws3(op_prop_t *o_p);
extern int16_t op_ws4(op_prop_t *o_p);
extern int16_t op_ws5(op_prop_t *o_p);
extern int16_t op_ws6(op_prop_t *o_p);
extern int16_t op_ws7(op_prop_t *o_p);

extern const int16_t stab[STAB_SIZE];
extern int16_t (* const op_ws_p[8])(op_prop_t *o_p);



#endif // OP_H