/****************************************************/
/*                                                  */
/* seq.h : SMAF / SMF sequencer define              */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/29 Rev. 1.2  support SMF sequencer       */
/* 2011/04/20 Rev. 1.1  by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef SEQ_H
#define SEQ_H

#define SMF_SB_METAEVENT    0xFF
#define SMF_METAEVENT_EOT   0x2F
#define SMF_METAEVENT_TEMPO 0x51
#define SMF_GET_BYTE(x) (dp->seq[seq_p++])

//
// SMF sequencer tick (4 ms) 
// in terms of micro-seconds
// (4 [ms] / 1 [us] = 4000)
//
#define SEQ_TICK_in_us (4000)
//
// SMF parameter Ticks Per Quater Note
// (normally 480, 960, etc.)
// 480 = 3 * 5 * 2**5 = 15 * 32
// 960 = 3 * 5 * 2**6 = 15 * 64
//
#define TPQN_FACTOR    (15)

//
// SMAF sequencer struct
//
typedef struct {
    int gatetime;   // remaining gatetime
    int octshift;   // octave shift
    int notenumber; // MIDI note number
  } seq_status_t ;

extern seq_status_t seq_st[MAXSEQCH];

//
// demo songs
//
typedef struct {
  const uint8_t *seq;
  const uint8_t inst[4];
        char    *title;
} demo_st;

extern const demo_st      demosong[];
extern const prog_uint8_t arabesq1[];
extern const prog_uint8_t arabe_sp[];
extern const prog_uint8_t i13_f0[];

#ifdef EXTRADEMO
extern const prog_uint8_t Chopin66[];
extern const prog_uint8_t invent13[];
#endif

//
// variables
//
extern int   demosave;
extern uint seq_duration;
extern int  seq_tick;
extern int  demoflag;
extern int  loopflag;
extern int  demo_no;
extern int  demosongs;
//
// for SMF sequencer
//
#if defined(USE_SMF)
extern int seq_mode;

extern  int32_t seq_time_inc;
extern  int32_t seq_targ_time;
extern uint32_t smf_tpqn; // SMF Ticks Per Quater Note
extern uint32_t smf_tempo_us; // SMF tempo in microseconds 
extern uint32_t mod_tpqn; // modified (factorized) TPQN
extern  int32_t mod_seq_tick_in_us; 
extern uint32_t seq_dt;
extern demo_st *dp;

#endif

//
// functions
//
extern int  seq_p;
extern void seq_songsel(int song_no);
extern void seq(void);
extern void demo_off(void);
extern void init_seq(void);
extern void seq_next_song(void);
extern void seq_songsel(int song_no);

extern int  get_dur  (const uint8_t *sq);
extern int  get_event(const uint8_t *sq);

#endif // SEQ_H