/****************************************************/
/*                                                  */
/* smf_seq.h : Standard MIDI File (SMF) sequencer   */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/29 Rev. 1.2  created by pcm1723          */
/*                                                  */
/****************************************************/
#include "../common/tonegen.h"

#if (defined(SEQUENCER) & defined(USE_SMF))

//
// MIDI message length table
//
const prog_uint8_t MD_len_tab[16] = {
// System messages  
  0, // 0xF0: System exclusive        = variable length
  2, // 0xF1: MTC quater frame        = 2 byte
  3, // 0xF2: Song Position Pointer   = 3 byte
  2, // 0xF3: Song Select             = 2 byte
  1, // 0xF4: undefined               = 1 byte
  2, // 0xF5: Port Select             = 2 byte
  1, // 0xF6: Tune Request            = 1 byte
  0, // 0xF7: End Of Exclusive
// Channel messages  
  3, // 0x8n: Note Off                = 3 byte
  3, // 0x9n: None On                 = 3 byte
  3, // 0xAn: Polyphonic Key Pressure = 3 byte
  3, // 0xBn: Control Change          = 3 byte
  2, // 0xCn: Program Change          = 2 byte
  2, // 0xDn: Channel Pressure        = 2 byte
  3, // 0xEn: Pitch Bend Change       = 3 byte
  0  // 0xFn: System messages
}; // const prog_uint8_t MD_len_tab[]


// SMF processing state 
enum {SMF_ST_HDR_CHUNK, SMF_ST_TRACK, SMF_ST_END_TRACK};

const uint8_t SMF_format0_hdr[] = {
  'M', 'T', 'h', 'd', 0, 0, 0, 6, 0, 0, 0, 1
}; // const uint8_t SMF_format0_hdr[]

const uint8_t SMF_MTrk_hdr[] = {
  'M', 'T', 'r', 'k'
}; // const uint8_t SMF_MTrk_hdr[]

int decode_midi_msg_len(uint8_t stbyte)
{
  if (0xF8 == (0xF8 & stbyte)) { // 0xF8..0xFF: System Realtime
    return(1);                   // 1 byte
  } // if (0xF8 == ..
  if (0xF0 == (0xF8 & stbyte)) { // 0xF0..0xF7: System Common / Exclusive
    return(MD_len_tab[0x07 & stbyte]);
  } else {                       // other messages
    return(MD_len_tab[stbyte >> 4]);
  } // if (0xF0 == ..
} // int decode_midi_msg_len()

uint32_t smf_get_delta_t( void )
{
  uint32_t dt = 0;
  uint8_t  b;
  do {
    b =  SMF_GET_BYTE();
    dt = (dt << 7) + (0x7f & b);
  } while (0x80 & b);
  return(dt);
} // uint32_t smf_get_delta_t()

void seq( void )
{
  int i;
  uint8_t  md_byte;
  
  while (  (SMF_ST_END_TRACK != seq_mode) 
         & (0 >= seq_targ_time) ) { //
    md_byte = SMF_GET_BYTE(); // the 1st byte of the event
    if (SMF_SB_METAEVENT == md_byte) { // SMF metaevent found
      md_byte = SMF_GET_BYTE(); // metaevent type
      i       = SMF_GET_BYTE(); // data length
      switch (md_byte) { 
        case SMF_METAEVENT_EOT: // End Of Track
          seq_mode = SMF_ST_END_TRACK; // signal END OF TRACK
          break;
        case SMF_METAEVENT_TEMPO: // Set Tempo metaevent
          smf_tempo_us = 0;
          for (; i > 0; i--) {
            smf_tempo_us <<= 8;
            smf_tempo_us  += SMF_GET_BYTE(); // tempo data bytes
          } // for (; i > 0; i--) ...
// compute sequencer tick increment (in modified micro-seconds) per SMF delta time tick          
          seq_time_inc = (smf_tempo_us  / mod_tpqn ) ;
          break;
        default: // skip other metaevents
          for (; i > 0; i--) { // for the event length
            md_byte = SMF_GET_BYTE(); // discard data byte(s)
          } // for (; i > 0; i--) ...
          break;
        } // switch (md_byte) { ...
    } else { // MIDI event
      i = decode_midi_msg_len(md_byte); // get length
      mididec(md_byte);                 // process STATUS BYTE
      if (0xf0 == md_byte) { // System Exclusive
         do { // repeat
           md_byte = SMF_GET_BYTE(); // data byte
           mididec(md_byte);         // process it
         } while (0xf7 != md_byte);  // until EOX (0xf7) found
      } else { // other message
        for (i = i - 1; i > 0; i--) { // for data byte length
          md_byte = SMF_GET_BYTE();   // get a data byte
          mididec(md_byte);           // process it
        } // for 
      } // if (0xf0 == md_byte) ...
    } // if (SMF_SB_METAEVENT == md_byte) ...
// advance (increment) sequencer tick to the next SMF event tick
    seq_dt = smf_get_delta_t();
    seq_targ_time += (seq_dt * seq_time_inc);
  } // while () ...
  if (SMF_ST_END_TRACK != seq_mode) { // normal
// advance (decrement) sequencer tick (4 ms) for next updates
    seq_targ_time -= mod_seq_tick_in_us;
  } else { // End Of Track found
// loop from first 
    if ((++demo_no) >= demosongs) {
      demo_no = 0; // reset to first song
      init_seq();
      demoflag = loopflag; // stop demo if not loop
      if (0 == demoflag) {
        demoflag = 1; // temporary set demoflag
        demo_off();   // demoflag cleared in demo_off()
      } // demo end
    } else { // if ((++demo_no) ...
      init_seq();
    } // if ((++demo_no) ... else ...
  } // if (SMF_ST_END_TRACK != seq_mode) ... else ...
} // void seq() 

void init_seq( void )
{
  int ch;
  dp = (demo_st *)&(demosong[demo_no]);
  GM_system_on();
#ifdef LCD
  lcd_clear(lcd_buf2);
  lcd_buf_p = lcd_buf2; // set sub screen
  lcd_buf_w = lcd_buf_p;
  lcd_putstr(dp->title);
  lcd_buf_w = lcd_buf; // restore forward screen
#endif
  for (ch = 0; ch < MAXMIDICH; ch++) {
    ch_prop_t *c_p = &(ch_prop[ch]);
    all_note_off(c_p);           // all note off 
    setup_voice(c_p, demosong[demo_no].inst[0]); // initialize voice data
  } // for (ch = 0; ...
  seq_mode = SMF_ST_HDR_CHUNK;
  seq_p  = 0;
// Verify "MThd" chunk  
  if (0 == memcmp(SMF_format0_hdr, dp->seq,
                  sizeof(SMF_format0_hdr))) { // correct header
    seq_p    += sizeof(SMF_format0_hdr); // skip header
    smf_tpqn  = (SMF_GET_BYTE() << 8); // TPQN high byte
    smf_tpqn +=  SMF_GET_BYTE();       // TPQN low  byte
// factorize TPQN    
    mod_tpqn           = smf_tpqn;
    mod_seq_tick_in_us = SEQ_TICK_in_us;
    if (0 == (smf_tpqn % TPQN_FACTOR)) { // TPQN factorizable
      mod_tpqn           /= TPQN_FACTOR;
      mod_seq_tick_in_us *= TPQN_FACTOR;
    } // if (0 == ...
  } else { // invalid header
    seq_mode = SMF_ST_END_TRACK;
  } // if (0 == memcmp(SMF_format0_hdr, ...
// Verify "MTrk" chunk  
  if (0 == memcmp(SMF_MTrk_hdr, dp->seq+seq_p,
                  sizeof(SMF_MTrk_hdr))) { // correct header
    seq_p += (sizeof(SMF_MTrk_hdr) + 4); // skip header
    seq_mode = SMF_ST_TRACK; // next, process track data
  } else { // invalid "MTrk" header
    seq_mode = SMF_ST_END_TRACK;
  } // if (0 == memcmp(SMF_MTrk_hdr, ...
  seq_targ_time = 0;
// get the very first SMF delta time  
  seq_dt = smf_get_delta_t();
  seq_targ_time += (seq_dt * seq_time_inc);
} // void init_seq()

void seq_songsel(int song_no)
{
  if ((song_no) >= demosongs) song_no = 0; // reset to first song
  else if (0 > song_no)       song_no = demosongs-1; // reset to last song
  demo_no = song_no;
  init_seq();
} // void songselect()

void seq_next_song(void)
{
  seq_songsel(demo_no+1);
} // void seq_next_song()

#endif
