/*************************************************************************/
/*   MIDI tone generator (FM synthesis) with demo song (SMAF sequencer)  */
/*   for CQ publishing CQ-FRK-RX62N board (Renesas RX62N)                */
/*   using HEW 4 environment                                             */
/*                                                                       */
/*   NON COMMERCIAL USE ONLY                                             */
/*                                                                       */
/*   2011/04/29 Rev. 1.2 support stereo (CC#10 PAN), SMF sequencer       */
/*   2011/04/24 Rev. 1.1 by pcm1723                                      */
/*                                                                       */
/*************************************************************************/

#include "tonegen.h"

//
// call machine specific port setup routine
//
void port_setup( void )
{
#if   defined(ADUC7026)
  arm_port_setup();
#elif defined(__v850__)
  v850_port_setup();
#elif defined(STM32F103)
  starm_port_setup();
#elif defined(LPC23XX)
  LPC23xx_port_setup();
#elif defined(_SH2AFPU)
  SH2A_port_setup();
#elif defined(__RX)
  RX62N_port_setup();
#endif
} // void port_setup()

//
// processing at every 4 ms interval
// (LFOs and portamento)
//
void rate_4ms_procs( void )
{
  int i;
  for (i = 0; i < MAXUNITS; i++) {
    update_LFO(&(ch_prop[i]));     // update each channel LFO at 4ms interval
    update_porta(&(unit_prop[i])); // update portament of each unit at 4ms
  } // if (MAXUNITS > t4ms)
#if (defined(SEQUENCER) & defined(USE_SMF))
  if (demoflag) {seq();} // SMF sequencer top level
#endif
} // void rate_4ms_procs()

//
// processing at every 1 ms interval, part-2
// for LCD refresh
//
void rate_1ms_procs2( void )
{
#if defined(LCD)
  switch (t4ms) { 
    case 3:  // force E trigger off
      lcd_E_down();
      if (lcd_disp_timer) { // LCD display timer working
        if (0 == --lcd_disp_timer) {
          lcd_buf_p = &(lcd_buf[0]); // restore pointer to normal screen
  #if defined(SEQUENCER)
          if (demosave) {
            demoflag = demosave;
            demosave = 0;
            init_seq();
          } // if (demosave)
  #endif
        } // if
      } // if 
      break;
  #if defined(LCD_4bit)
    case 2: // LCD low nibble output timing
      lcd_refresh(1);
      break;
    case 1: // force E trigger off
      lcd_E_down();
      break;
  #endif
    case 0: // LCD high nibble output timing
      lcd_refresh(0);
      break;
    default: break;
  } // switch 
#endif
} // void rate_1ms_procs2()

//
// processing at every 1 ms interval, part-1
//
void rate_1ms_procs( void )
{
#if defined(LCD)
  if (0 > --lcd_blink_timer) { // LCD blink timer
    lcd_blink_timer = LCD_BLINK_ms - 1;
    if (&(lcd_buf[0]) == lcd_buf_p) {
      lcd_blink_flag  ^= 1; // flip blink flag
    } else {                // sub screen displayed
      lcd_blink_flag   = 0; // do not blink if so
    } // else
  } // if (0 > --lcd_blink_timer) 
#endif
  t4ms++;
  t4ms &= 0x3; // divide by 4 with 1ms tick
  if (0 == t4ms) {rate_4ms_procs();}
#if defined(LCD)
  if (0 >= --debounce) { // debounce interval
    debounce = DEBOUNCE_ms; // restore debounce counter
    if ((sw_val = sw_input())) {  // INC or DEC switch released
  #if defined(SEQUENCER)
      if (demoflag | demosave) {     // in demo mode
        switch (abs(sw_val)) { //
          case CLICK :    // skip to next song
            seq_songsel(demo_no+sw_val);
            break;
          case PRESS :    // exit from demo mode
            demo_off();
            break;
          default: break;
        } // switch
      } // if (demoflag)
      else 
  #endif
        panel_cmd(sw_val); // other commands
    } else { // if ((sw_val == sw_input())) ...
      sw_val = NOCHANGE;
      if (SET_CONFIG == panel_mode) lcd_cpuload();
    } // if ((sw_val == sw_input())) ... else ...
  } // if (0 >= debounce)
#endif      
} // rate_1ms_procs()

//
// tone generator main
//
int tg_main(void)
{
  volatile uint16_t count=0;
#if !defined(USEWAVEQUE)
  uint32_t lw;
#endif
	long int w0;
#if defined(REVERB)
  volatile long w1;
#endif
  int i, j;
  volatile long int sum;
  volatile int32_t dry_l, dry_r;

#if defined(SEQUENCER)
  demosave = demoflag;
  demo_no = 0;
  #if defined(LCD)
  demoflag = 0; 
  #endif
#endif            

#if (0)
// search for a configuration with fs >= 15 kHz
  for (i  = MAX_CONFIG - 1;
       (15 > sys_config[fastchip][i].tick1ms) & (0 < i);
       i--) {} 
//  sys_mode_index = i; 
#ifdef DEMOONLY
  sys_mode_index = 0;
#else
  sys_mode_index = 3;
#endif
#endif // #if (0)

  sys_mode_index = 1;
  
  last_note = -1;
  midi_mon_part = 0; // MIDI monitor channel
  wq_wr_ix = 0; // wave queue write index = 0
  wq_rd_ix = 0; // wave queue read  index = 0
// fill up wave queue with 0
  for (i = 0; i < WAVEQUESIZE; i++) {
    for (j = 0; j <= STEREO_MODE; j++) {
      wave_que[i][j] = 0;
    } // for (j = 0; ...
  } // for (i = 0; ...
#if defined(REVERB)
// fill up delay queue with 0
  for (i = 0; i < DELAYMEMSIZE; i++) {
    delaymem[i] = 0;
  }  // for (i = 0; ...
  delayp = 0;
  echo_dep = 128;
#endif

  port_setup(); // hardware setup                   
  set_config(sys_mode_index);
  hard_reset(); // tone gererator hard reset
  enable_interrupt();
  
#if defined(LCD)
  panel_mode = MIDI_MON;
  lcd_init();
  lcd_draw_panel(panel_mode);
  lcd_buf_ind = 0;
#elif defined(SEQUENCER)
  init_seq();
#endif

  w0  = 0;
  sum = 0; // clear initial wave output
  wq_wr_ix = 0; // wave queue write index = 0
  wq_rd_ix = 0; // wave queue read  index = 0
  while (1) { // Main sampling rate loop
    if (CHECKMIDI) { // serial received
      uint8_t temp;
      LED_OFF;
      temp = GETMIDI;        
#if defined(SEQUENCER)      
      if (demoflag && (0xf0 == temp)) { // SysX received while demo mode
        demo_off();
      } // if (demoflag && ...
#endif      
      mididec(temp); // decode MIDI data
    } else {// if (CHECKMIDI) ...
      if (wq_wr_ix != wq_rd_ix) { // wave queue has room to write
#if (STEREO_MODE)
        wave_que[wq_wr_ix][0] = (w0 >> 1) + (dry_l >> (15-UNIT_VAL_SHIFT+2))/*sum*/;
        wave_que[wq_wr_ix][1] = (w0 >> 1) + (dry_r >> (15-UNIT_VAL_SHIFT+2))/*sum*/;
#else        
  #if defined(REVERB)
        sum += (w0 >> 1); // mix dry 50% : reverb 50 %
  #endif
        wave_que[wq_wr_ix][0] = sum;
#endif        
        wq_wr_ix = ((wq_wr_ix + 1) & (WAVEQUESIZE-1));
        t1_tick =0;   // clear T1 tick flag
        if (1 == tick1ms) { // EG update timing
          for (i = 0; i < MAXUNITS; i++) {
            unit_prop_t *u_p = &(unit_prop[i]);
            eg(u_p, &(u_p->op_prop[0]));
            eg(u_p, &(u_p->op_prop[1]));
          } // for ( i = 0; ...
        } // if (1 == tick1ms) ...
        LED_OFF;    
// combine output of units with 1/4 scaling to avoid overflow
        sum= 0;
#if (STEREO_MODE)        
        dry_l = UNIT_VAL_ROUND;
        dry_r = UNIT_VAL_ROUND;
#endif        
        for (i = 0; i < MAXUNITS; i++) {
          int32_t unit_val;
#if (STEREO_MODE)          
          int pan_val = ch_prop[i].MD_pan;
#endif          
//          int32_t ssum = sum;
          unit_val = unit(&(unit_prop[i])); 
            sum += (unit_val >> 2); 
#if (STEREO_MODE)          
            dry_l += ((cc10_tab[    pan_val] * unit_val) >> UNIT_VAL_SHIFT);
            dry_r += ((cc10_tab[127-pan_val] * unit_val) >> UNIT_VAL_SHIFT);
#endif          
        } // for (i = 0; ...
#if defined(REVERB)
        w0 = delaymem[delayp]; // read sample from delay memory
        w1 = ((w0) >> 1)                            // feedback of delayed sample
           + ((sum * ch_prop[0].MD_rev_send) >> 7); // mult by reverb send level of ch0
// too high send level may cause overflow, apply clipping if so
        if      ( MAXSHORT < w1) w1 =  MAXSHORT;
        else if (-MAXSHORT > w1) w1 = -MAXSHORT;
        delaymem[delayp++] = -w1; // write sample in delay memory
        if (delay_mem_size <= delayp) delayp = 0; // modulo operation
#endif  // #if devined(REVERB)
        if (0 >= --tick1ms) { // 1 ms tick timing
          tick1ms = SEQTICK1ms; // restore 1ms counter
          rate_1ms_procs();
        } // if (0 >= --tick1ms)
#if defined(LCD)
        if (4 == tick1ms) {rate_1ms_procs2();}
#endif
#if (defined(SEQUENCER) & !defined(USE_SMF))
        if (demoflag) {seq();} // SMAF sequencer top level
#endif
        LED_ON;       
      }  // if (wq_wr_ix != wq_rd_ix) { ...
    } // if (CHECKMIDI) {} else { ...
  } // while (1) 
} // int tg_main()
