#ifndef TGMISC2_H
#define TGMISC2_H

#if  defined(ADUC7026)
  #define LED_OFF (GP4DAT |=  0x00800000) 
  #define LED_ON  (GP4DAT &= ~0x00800000)
  #define CHECKMIDI  (0x01==(0x01 & COMSTA0))
  #define GETMIDI    (COMRX)
  #define PUTMIDI(x) (COMTX = (x))
  #define P0_PWM2H (0x01 << 4)
  #define P0_PWM2L (0x01 << 8)
  #define P0_SIN   (0x02 << 28)
  #define P1_SIN   (0x01)
  #define P1_SOUT  (0x01 << 4)
  #define P2_SOUT  (0x02)
  #define GET_PANEL_KEY (0x01 & GP0DAT)
  extern void arm_port_setup(void);
  #define DELAYMEMSIZE ((6400 - UNIT_RAM_SIZE) >> 1)
  #define SMIDI_SENS_PORT    GP4DAT
  #define SMIDI_SENS_BITMASK 0x08
#elif defined(__v850__)
  #define LED_OFF (PCT.6 = 1, P7H.1 = 1)
  #define LED_ON  (PCT.6 = 0, P7H.1 = 0)
  #define GETMIDI (UA2RIF = 0, UA2RX)
  #define CHECKMIDI (1 == UA2RIF)
  #define PUTMIDI(x) (UA2TX = x)
  #define GET_PANEL_KEY (0x01)
  #define DELAYMEMSIZE (4900)
#elif defined(STM32F103)
  #ifdef __IAR__
    #define LED_OFF (GPIOC_ODR_bit.ODR6 = 0)
    #define LED_ON  (GPIOC_ODR_bit.ODR6 = 1)
    #define CHECKMIDI (USART1_SR_bit.RXNE)
    #define GETMIDI (USART1_DR)
    #define PUTMIDI(x) (USART1_DR=x)
  #else // #ifdef __IAR__
    #define LED_OFF (GPIOC->ODR = (GPIOC->ODR & 0xFFFFFFBF))
    #define LED_ON  (GPIOC->ODR = (GPIOC->ODR & 0xFFFFFFBF) | (1 << 6))
    #ifdef DEMOONLY
      #define CHECKMIDI (0x00)
    #else
      #define CHECKMIDI ((1<<5) & USART1->SR)
    #endif // #ifdef DEMOONLY
    #define GETMIDI (USART1->DR)
    #define PUTMIDI(x) (USART1->DR=x)
  #endif // #ifdef __IAR__ ... #else
  #define GET_PANEL_KEY (0x01)
  #define DELAYMEMSIZE (4800)
  extern void starm_port_setup( void );
#elif defined(LPC23XX)
  #define LED_OFF (FIO1SET_bit.P1_18 = 1)
  #define LED_ON  (FIO1CLR_bit.P1_18 = 1)
  #ifdef DEMOONLY
    #define CHECKMIDI (0x00)
  #else
    #define CHECKMIDI (U1LSR_bit.DR)
  #endif // #ifdef DEMOONLY 
  #define GETMIDI (U1RBR)
  #define PUTMIDI(x) (U1THR=x)
  #define GET_PANEL_KEY (0x01)
  #define DEFAULT_DELAY_ms 120
  #define DELAYMEMSIZE (48*DEFAULT_DELAY_ms)
  extern void LPC23xx_port_setup( void );
#else
#endif

#if defined(syn_CV)
  #if defined(__v850__)
    #define SPICLK_kHz 250
    #define SPIQUESIZE 16
    extern volatile int spiq_wr_ix, spiq_rd_ix;
    extern int8_t spi_que[SPIQUESIZE];
    #define CHECK_SPIQ   (spiq_wr_ix != spiq_rd_ix)
    #define PUT_SPIQ(x)  (spi_que[spiq_wr_ix++] = (x), spiq_wr_ix &= (SPIQUESIZE-1))
    #define XMIT_SPI     (CB3TX = spi_que[spiq_rd_ix++], spiq_rd_ix &= (SPIQUESIZE-1))
    #define SPI_1ST_BYTE (0 == (0x01 & spiq_rd_ix))
    #define SS_HIGH      (P9H.2 = 0x01)
    #define SS_LOW       (P9H.2 = 0x00)
    #define GATE_ON      (P9H.5 = 0x01)
    #define GATE_OFF     (P9H.5 = 0x00)
    #define syn_CV_ofs   (152)
    #define PDMWIDTH 42
    #define CALC_PWMVAL(x) ((x - PDMWIDTH)>>1)
  #endif
#endif

#endif // TGMISC2_H